<?php

/**
 * This is an example simple XML-RPC client which requires PHP Curl and PHP
 * XML-RPC libraries installed.  This client can be used to test the WordPress
 * XML-RPC Example Plugin, or as a template for creating custom code to interact
 * with WordPress over XML-RPC.
 * 
 * To run this script, perform the following:
 * 
 * 1. Create an XML RPC user with at least the Contributor role, and set the 
 *    username/password where defined below
 * 2. Configure the XMLRPC_ENDPOINT below with the URL to the xmlrpc.php file
 *    in the root of your WordPress install
 * 3. Ensure the XML-RPC service is enabled for your WordPress install by going
 *    to Settings > Writing > Remote Publishing, and checking the box next to XML-RPC
 * 4. Execute this script, for instance from the command line:
 *    $ php xmlrpc-client.php
 * 
 * 
 * 	Copyright:  2012 Justin Stern (email : justin@foxrunsoftware.net)
 *	License: GNU General Public License v3.0
 *	License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

/**
 * XML-RPC endpoint URL.  If your WordPress root is http://example.com/wordpress/
 * this value should be http://example.com/wordpress/xmlrpc.php
 */
define( 'XMLRPC_ENDPOINT', "http://127.0.0.1/xmlrpc.php" );
/**
 * Not currently used, but must be passed
 */
define( 'BLOG_ID', 1 );
/**
 * XML-RPC user, must be at least a contributor in order to set order tracking
 */
define( 'USERNAME', 'xmlrpc-user' );
/**
 * Password for the XML-RPC user
 */
define( 'PASSWORD', 'xmlrpc-user' );

/**
 * Example XML-RPC client
 */
class XMLRPC_Client {
	private $url;
	
	function __construct( $url ) {
		$this->url = $url;
	}
	
	/**
	 * Call the XML-RPC method named $method and return the results, or die trying!
	 * 
	 * @param string $method XML-RPC method name
	 * @param mixed ... optional variable list of parameters to pass to XML-RPC call
	 * 
	 * @return array result of XML-RPC call
	 */
	public function call() {
		// get arguments
		
		$params = func_get_args();
		$method = array_shift( $params );
		
		$post = xmlrpc_encode_request( $method, $params );
		
		$ch = curl_init();
		
		// set URL and other appropriate options
		curl_setopt( $ch, CURLOPT_URL,            $this->url );
		curl_setopt( $ch, CURLOPT_POST,           true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS,     $post );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		
		// issue the request
		$response = curl_exec( $ch );
		$response_code = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
		
		$curl_errorno = curl_errno( $ch );
		$curl_error   = curl_error( $ch );
		
		curl_close( $ch );
		
		// check for curl errors
		if ( $curl_errorno != 0 ) {
			die( "Curl ERROR: {$curl_errorno} - {$curl_error}\n" );
		}
		
		// check for server errors
		if ( $response_code != 200 ) {
			die( "ERROR: non-200 response from server: {$response_code} - {$response}\n" );
		}
		
		return xmlrpc_decode( $response );
	}
}

// 1. create the client
$client = new XMLRPC_Client( XMLRPC_ENDPOINT );


// 2. retrieve the list of available methods
echo "Checking for method availability:             ";
$available_methods = $client->call( 'system.listMethods' );

if ( ! is_array( $available_methods ) || ! in_array( "frs.helloWorld", $available_methods ) ) {
	echo "ERROR - frs.helloWorld not found as an available method, are you sure you installed and activated the 'WordPress XML-RPC Demo' plugin?\n";
	exit;
}

echo "Found!\n";

// 3. test authentication/permission
echo "Verifying authentication/permission for user: ";
$resp = $client->call( 'wp.getCommentCount', BLOG_ID, USERNAME, PASSWORD );

if ( isset ( $resp['faultCode'] ) ) {
	switch ( $resp['faultCode'] ) {
		case 403: die( "ERROR: {$resp['faultString']}" );  // bad user/pass or permissions for user are too low
		case 405: die( "ERROR: {$resp['faultString']}" );  // XML-RPC services are disabled
		default:  die( "ERROR: {$resp['faultCode']} - {$resp['faultString']}" );
	}
}

echo "SUCCESS ({$resp['total_comments']} comments in your blog)\n";


// 4. An example call to helloWorld
echo "Testing frs.helloWorld call:                  ";

$resp = $client->call( 'frs.helloWorld', BLOG_ID, USERNAME, PASSWORD,
						array( 'name' => 'Justin' ) );

var_dump( $resp );
